@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2019 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2020 Savoir-faire Linux Inc.
  *
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *
@@ -35,6 +35,7 @@ import android.content.pm.PackageManager;
 import android.graphics.Typeface;
 import android.content.res.Resources;
 import android.os.Bundle;
+import android.os.Environment;
 import android.os.IBinder;
 import android.provider.MediaStore;
 import android.text.Editable;
@@ -75,7 +76,6 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
-import butterknife.OnEditorAction;
 import cx.ring.BuildConfig;
 import cx.ring.R;
 import cx.ring.adapters.ConversationAdapter;
@@ -89,7 +89,6 @@ import cx.ring.contacts.AvatarFactory;
 import cx.ring.conversation.ConversationPresenter;
 import cx.ring.conversation.ConversationView;
 import cx.ring.databinding.FragConversationBinding;
-import cx.ring.dependencyinjection.JamiInjectionComponent;
 import cx.ring.interfaces.Colorable;
 import cx.ring.model.Account;
 import cx.ring.model.CallContact;
@@ -102,6 +101,7 @@ import cx.ring.model.Uri;
 import cx.ring.mvp.BaseSupportFragment;
 import cx.ring.services.LocationSharingService;
 import cx.ring.services.NotificationService;
+import cx.ring.services.NotificationServiceImpl;
 import cx.ring.utils.ActionHelper;
 import cx.ring.utils.AndroidFileUtils;
 import cx.ring.utils.ContentUriHandler;
@@ -155,6 +155,8 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     private final CompositeDisposable mCompositeDisposable = new CompositeDisposable();
     private int mSelectedPosition;
 
+    private boolean mIsBubble;
+
     private AvatarDrawable mConversationAvatar;
     private final Map<String, AvatarDrawable> mParticipantAvatars = new HashMap<>();
     private final Map<String, AvatarDrawable> mSmallParticipantAvatars = new HashMap<>();
@@ -197,16 +199,6 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         }
     }
 
-    @Override
-    public int getLayout() {
-        return R.layout.frag_conversation;
-    }
-
-    @Override
-    public void injectFragment(JamiInjectionComponent component) {
-        component.inject(this);
-    }
-
     private static void setBottomPadding(@NonNull View view, int padding) {
         view.setPadding(
                 view.getPaddingLeft(),
@@ -227,7 +219,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     @Nullable
     @Override
     public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
-        injectFragment(((JamiApplication) getActivity().getApplication()).getRingInjectionComponent());
+        ((JamiApplication) getActivity().getApplication()).getInjectionComponent().inject(this);
         Resources res = getResources();
         marginPx = res.getDimensionPixelSize(R.dimen.conversation_message_input_margin);
         mapWidth = res.getDimensionPixelSize(R.dimen.location_sharing_minmap_width);
@@ -315,7 +307,6 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     @Override
     public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
         super.onViewCreated(view, savedInstanceState);
-
         if (mPreferences != null) {
             String pendingMessage = mPreferences.getString(KEY_PREFERENCE_PENDING_MESSAGE, null);
             if (!TextUtils.isEmpty(pendingMessage)) {
@@ -366,6 +357,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         }
         mAdapter = null;
         super.onDestroyView();
+        binding = null;
     }
 
     @Override
@@ -392,10 +384,6 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         presenter.sendTextMessage(binding.emojiSend.getText().toString());
     }
 
-    public void selectFile() {
-        presenter.selectFile();
-    }
-
     @SuppressLint("RestrictedApi")
     public void expandMenu(View v) {
         Context context = requireContext();
@@ -403,42 +391,14 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         popup.inflate(R.menu.conversation_share_actions);
         popup.setOnMenuItemClickListener(item -> {
             switch(item.getItemId()) {
-                case R.id.conv_send_audio: {
-                    Intent intent = new Intent(MediaStore.Audio.Media.RECORD_SOUND_ACTION);
-                    if (intent.resolveActivity(context.getPackageManager()) != null) {
-                        try {
-                            mCurrentPhoto = AndroidFileUtils.createAudioFile(context);
-                        } catch (IOException ex) {
-                            Log.e(TAG, "takePicture: error creating temporary file", ex);
-                            break;
-                        }
-                        startActivityForResult(intent, REQUEST_CODE_CAPTURE_AUDIO);
-                    } else {
-                        Toast.makeText(getActivity(), "Can't find audio recorder app", Toast.LENGTH_SHORT).show();
-                    }
+                case R.id.conv_send_audio:
+                    sendAudioMessage();
                     break;
-                }
-                case R.id.conv_send_video: {
-                    Intent intent = new Intent(MediaStore.ACTION_VIDEO_CAPTURE);
-                    intent.putExtra("android.intent.extras.CAMERA_FACING", android.hardware.Camera.CameraInfo.CAMERA_FACING_FRONT);
-                    intent.putExtra("android.intent.extras.LENS_FACING_FRONT", 1);
-                    intent.putExtra("android.intent.extra.USE_FRONT_CAMERA", true);
-                    if (intent.resolveActivity(context.getPackageManager()) != null) {
-                        try {
-                            mCurrentPhoto = AndroidFileUtils.createVideoFile(context);
-                        } catch (IOException ex) {
-                            Log.e(TAG, "takePicture: error creating temporary file", ex);
-                            break;
-                        }
-                        intent.putExtra(MediaStore.EXTRA_OUTPUT, ContentUriHandler.getUriForFile(context, ContentUriHandler.AUTHORITY_FILES, mCurrentPhoto));
-                        startActivityForResult(intent, REQUEST_CODE_CAPTURE_VIDEO);
-                    } else {
-                        Toast.makeText(getActivity(), "Can't find video recorder app", Toast.LENGTH_SHORT).show();
-                    }
+                case R.id.conv_send_video:
+                    sendVideoMessage();
                     break;
-                }
                 case R.id.conv_send_file:
-                    selectFile();
+                    presenter.selectFile();
                     break;
                 case R.id.conv_share_location:
                     shareLocation();
@@ -483,7 +443,6 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
 
     /**
      * Used to update with the past adapter position when a long click was registered
-     * @param position
      */
     public void updatePosition(int position) {
         mSelectedPosition = position;
@@ -525,35 +484,74 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         }
     }
 
-    public void takePicture() {
-        if (!presenter.getDeviceRuntimeService().hasVideoPermission()) {
-            requestPermissions(new String[]{Manifest.permission.CAMERA}, JamiApplication.PERMISSIONS_REQUEST);
+    public void sendAudioMessage() {
+        if (!presenter.getDeviceRuntimeService().hasAudioPermission()) {
+            requestPermissions(new String[]{Manifest.permission.RECORD_AUDIO}, REQUEST_CODE_CAPTURE_AUDIO);
         } else {
-            Context c = getContext();
-            if (c == null)
-                return;
-            Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
-            if (takePictureIntent.resolveActivity(c.getPackageManager()) != null) {
-                // Create the File where the photo should go
-                File photoFile;
+            Context ctx = requireContext();
+            Intent intent = new Intent(MediaStore.Audio.Media.RECORD_SOUND_ACTION);
+
+            if (intent.resolveActivity(ctx.getPackageManager()) != null) {
                 try {
-                    photoFile = AndroidFileUtils.createImageFile(c);
+                    mCurrentPhoto = AndroidFileUtils.createAudioFile(ctx);
                 } catch (IOException ex) {
                     Log.e(TAG, "takePicture: error creating temporary file", ex);
                     return;
                 }
-                Log.i(TAG, "takePicture: trying to save to " + photoFile);
-                mCurrentPhoto = photoFile;
-                android.net.Uri photoURI = ContentUriHandler.getUriForFile(c, ContentUriHandler.AUTHORITY_FILES, photoFile);
-                takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI);
-                takePictureIntent.putExtra("android.intent.extras.CAMERA_FACING", 1);
-                takePictureIntent.putExtra("android.intent.extras.LENS_FACING_FRONT", 1);
-                takePictureIntent.putExtra("android.intent.extra.USE_FRONT_CAMERA", true);
-                startActivityForResult(takePictureIntent, REQUEST_CODE_TAKE_PICTURE);
+                startActivityForResult(intent, REQUEST_CODE_CAPTURE_AUDIO);
+            } else {
+                Toast.makeText(getActivity(), "Can't find audio recorder app", Toast.LENGTH_SHORT).show();
             }
         }
     }
 
+    public void sendVideoMessage() {
+        if (!presenter.getDeviceRuntimeService().hasVideoPermission()) {
+            requestPermissions(new String[]{Manifest.permission.CAMERA}, REQUEST_CODE_CAPTURE_VIDEO);
+        } else {
+            Context context = requireContext();
+            Intent intent = new Intent(MediaStore.ACTION_VIDEO_CAPTURE);
+            intent.putExtra("android.intent.extras.CAMERA_FACING", android.hardware.Camera.CameraInfo.CAMERA_FACING_FRONT);
+            intent.putExtra("android.intent.extras.LENS_FACING_FRONT", 1);
+            intent.putExtra("android.intent.extra.USE_FRONT_CAMERA", true);
+            if (intent.resolveActivity(context.getPackageManager()) != null) {
+                try {
+                    mCurrentPhoto = AndroidFileUtils.createVideoFile(context);
+                } catch (IOException ex) {
+                    Log.e(TAG, "takePicture: error creating temporary file", ex);
+                    return;
+                }
+                intent.putExtra(MediaStore.EXTRA_OUTPUT, ContentUriHandler.getUriForFile(context, ContentUriHandler.AUTHORITY_FILES, mCurrentPhoto));
+                startActivityForResult(intent, REQUEST_CODE_CAPTURE_VIDEO);
+            } else {
+                Toast.makeText(getActivity(), "Can't find video recorder app", Toast.LENGTH_SHORT).show();
+            }
+        }
+    }
+
+    public void takePicture() {
+        if (!presenter.getDeviceRuntimeService().hasVideoPermission()) {
+            requestPermissions(new String[]{Manifest.permission.CAMERA}, REQUEST_CODE_TAKE_PICTURE);
+            return;
+        }
+        Context c = getContext();
+        if (c == null)
+            return;
+        try {
+            File photoFile = AndroidFileUtils.createImageFile(c);
+            Log.i(TAG, "takePicture: trying to save to " + photoFile);
+            android.net.Uri photoURI = ContentUriHandler.getUriForFile(c, ContentUriHandler.AUTHORITY_FILES, photoFile);
+            Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE).putExtra(MediaStore.EXTRA_OUTPUT, photoURI)
+                    .putExtra("android.intent.extras.CAMERA_FACING", 1)
+                    .putExtra("android.intent.extras.LENS_FACING_FRONT", 1)
+                    .putExtra("android.intent.extra.USE_FRONT_CAMERA", true);
+            mCurrentPhoto = photoFile;
+            startActivityForResult(takePictureIntent, REQUEST_CODE_TAKE_PICTURE);
+        } catch (Exception e) {
+            Toast.makeText(c, "Error taking picture: " + e.getLocalizedMessage(), Toast.LENGTH_SHORT).show();
+        }
+    }
+
     @Override
     public void askWriteExternalStoragePermission() {
         requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, JamiApplication.PERMISSIONS_REQUEST);
@@ -583,24 +581,22 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     }
 
     @Override
-    public void onActivityResult(int requestCode, int resultCode, Intent resultData) {
+    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent resultData) {
         Log.w(TAG, "onActivityResult: " + requestCode + " " + resultCode + " " + resultData);
-        super.onActivityResult(requestCode, resultCode, resultData);
-
-        if (requestCode == REQUEST_CODE_FILE_PICKER && resultCode == RESULT_OK) {
-            if (resultData == null) {
-                return;
+        if (resultData == null) {
+            return;
+        }
+        android.net.Uri uri = resultData.getData();
+        if (requestCode == REQUEST_CODE_FILE_PICKER) {
+            if (resultCode == RESULT_OK && uri != null) {
+                startFileSend(AndroidFileUtils.getCacheFile(requireContext(), uri)
+                        .observeOn(AndroidSchedulers.mainThread())
+                        .flatMapCompletable(this::sendFile));
             }
-            android.net.Uri uri = resultData.getData();
-            if (uri == null) {
-                return;
-            }
-            startFileSend(AndroidFileUtils.getCacheFile(requireContext(), uri)
-                    .observeOn(AndroidSchedulers.mainThread())
-                    .flatMapCompletable(this::sendFile));
         } else if (requestCode == REQUEST_CODE_TAKE_PICTURE
                 || requestCode == REQUEST_CODE_CAPTURE_AUDIO
-                || requestCode == REQUEST_CODE_CAPTURE_VIDEO) {
+                || requestCode == REQUEST_CODE_CAPTURE_VIDEO)
+        {
             if (resultCode != RESULT_OK) {
                 mCurrentPhoto = null;
                 return;
@@ -608,9 +604,8 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
             Log.w(TAG, "onActivityResult: mCurrentPhoto " + mCurrentPhoto.getAbsolutePath() + " " + mCurrentPhoto.exists() + " " + mCurrentPhoto.length());
             Single<File> file = null;
             if (mCurrentPhoto == null || !mCurrentPhoto.exists() || mCurrentPhoto.length() == 0) {
-                android.net.Uri createdUri = resultData.getData();
-                if (createdUri != null) {
-                    file = AndroidFileUtils.getCacheFile(requireContext(), createdUri);
+                if (uri != null) {
+                    file = AndroidFileUtils.getCacheFile(requireContext(), uri);
                 }
             } else {
                 file = Single.just(mCurrentPhoto);
@@ -623,38 +618,42 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
             startFileSend(file.flatMapCompletable(this::sendFile));
         }
         // File download trough SAF
-        else if(requestCode == ConversationFragment.REQUEST_CODE_SAVE_FILE
-                && resultCode == RESULT_OK){
-            if(resultData != null && resultData.getData() != null ) {
-                //Get the Uri of the file that was created by the app that received our intent
-                android.net.Uri createdUri = resultData.getData();
-
-                //Try to copy the data of the current file into the newly created one
-                File input = new File(mCurrentFileAbsolutePath);
-                if(requireContext().getContentResolver() != null)
-                    AndroidFileUtils.copyFileToUri(
-                            requireContext().getContentResolver(),input,createdUri).
-                            observeOn(AndroidSchedulers.mainThread()).
-                            subscribe(()->Toast.makeText(getContext(), R.string.file_saved_successfully,
-                                    Toast.LENGTH_SHORT).show(),
-                                    error->Toast.makeText(getContext(), R.string.generic_error,
-                                            Toast.LENGTH_SHORT).show());
-
+        else if (requestCode == ConversationFragment.REQUEST_CODE_SAVE_FILE) {
+            if (resultCode == RESULT_OK &&  uri != null) {
+                writeToFile(uri);
             }
         }
+    }
 
+    private void writeToFile(android.net.Uri data) {
+        File input = new File(mCurrentFileAbsolutePath);
+        if (requireContext().getContentResolver() != null)
+            mCompositeDisposable.add(AndroidFileUtils.copyFileToUri(requireContext().getContentResolver(), input, data).
+                    observeOn(AndroidSchedulers.mainThread()).
+                    subscribe(() -> Toast.makeText(getContext(), R.string.file_saved_successfully, Toast.LENGTH_SHORT).show(),
+                            error -> Toast.makeText(getContext(), R.string.generic_error, Toast.LENGTH_SHORT).show()));
     }
 
     @Override
     public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
-        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
         for (int i = 0, n = permissions.length; i < n; i++) {
+            boolean granted = grantResults[i] == PackageManager.PERMISSION_GRANTED;
             switch (permissions[i]) {
                 case Manifest.permission.CAMERA:
-                    boolean granted = grantResults[i] == PackageManager.PERMISSION_GRANTED;
                     presenter.cameraPermissionChanged(granted);
                     if (granted) {
-                        takePicture();
+                        if (requestCode == REQUEST_CODE_CAPTURE_VIDEO) {
+                            sendVideoMessage();
+                        } else if (requestCode == REQUEST_CODE_TAKE_PICTURE) {
+                            takePicture();
+                        }
+                    }
+                    return;
+                case Manifest.permission.RECORD_AUDIO:
+                    if (granted) {
+                        if (requestCode == REQUEST_CODE_CAPTURE_AUDIO) {
+                            sendAudioMessage();
+                        }
                     }
                     return;
                 default:
@@ -741,7 +740,6 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         }
     }
 
-    @OnEditorAction(R.id.msg_input_txt)
     boolean actionSendMsgText(int actionId) {
         switch (actionId) {
             case EditorInfo.IME_ACTION_SEND:
@@ -764,7 +762,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     @Override
     public void onResume() {
         super.onResume();
-        presenter.resume();
+        presenter.resume(mIsBubble);
     }
 
     @Override
@@ -805,15 +803,16 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
 
     @Override
     protected void initPresenter(ConversationPresenter presenter) {
-        Bundle args = getArguments();
-        if (args == null)
+        ConversationPath path = ConversationPath.fromBundle(getArguments());
+        mIsBubble = getArguments().getBoolean(NotificationServiceImpl.EXTRA_BUBBLE);
+        if (path == null)
             return;
-        Uri contactUri = new Uri(args.getString(KEY_CONTACT_RING_ID));
-        String accountId = args.getString(KEY_ACCOUNT_ID);
+
+        Uri contactUri = path.getConversationUri();
         mAdapter = new ConversationAdapter(this, presenter);
-        presenter.init(contactUri, accountId);
+        presenter.init(contactUri, path.getAccountId());
         try {
-            mPreferences = requireActivity().getSharedPreferences(accountId + "_" + contactUri.getRawRingId(), Context.MODE_PRIVATE);
+            mPreferences = requireActivity().getSharedPreferences(path.getAccountId() + "_" + contactUri.getRawRingId(), Context.MODE_PRIVATE);
             mPreferences.registerOnSharedPreferenceChangeListener(this);
             presenter.setConversationColor(mPreferences.getInt(KEY_PREFERENCE_CONVERSATION_COLOR, getResources().getColor(R.color.color_primary_light)));
         } catch (Exception e) {
@@ -829,7 +828,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
                     LocationSharingService locationService = binder.getService();
                     ConversationPath path = new ConversationPath(presenter.getPath());
                     if (locationService.isSharing(path)) {
-                        showMap(accountId, contactUri.getUri(), false);
+                        showMap(path.getAccountId(), contactUri.getUri(), false);
                     }
                     try {
                         requireContext().unbindService(locationServiceConnection);
@@ -935,9 +934,9 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     }
 
     @Override
-    public void goToContactActivity(String accountId, String contactRingId) {
-        startActivity(new Intent(Intent.ACTION_VIEW, android.net.Uri.withAppendedPath(android.net.Uri.withAppendedPath(ContentUriHandler.CONTACT_CONTENT_URI, accountId), contactRingId))
-                .setClass(requireActivity().getApplicationContext(), ContactDetailsActivity.class));
+    public void goToContactActivity(String accountId, String contactId) {
+        startActivity(new Intent(Intent.ACTION_VIEW, ConversationPath.toUri(accountId, contactId),
+                requireActivity().getApplicationContext(), ContactDetailsActivity.class));
     }
 
     @Override
@@ -1139,37 +1138,38 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         //Get the current file absolute path and store it
         mCurrentFileAbsolutePath = currentFileAbsolutePath;
 
-        //Use Android Storage File Access to download the file
-        Intent downloadFileIntent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
-        downloadFileIntent.setType(AndroidFileUtils.getMimeTypeFromExtension(file.getExtension()));
-        downloadFileIntent.addCategory(Intent.CATEGORY_OPENABLE);
-        downloadFileIntent.putExtra(Intent.EXTRA_TITLE,file.getDisplayName());
+        try {
+            //Use Android Storage File Access to download the file
+            Intent downloadFileIntent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
+            downloadFileIntent.setType(AndroidFileUtils.getMimeTypeFromExtension(file.getExtension()));
+            downloadFileIntent.addCategory(Intent.CATEGORY_OPENABLE);
+            downloadFileIntent.putExtra(Intent.EXTRA_TITLE,file.getDisplayName());
 
-        startActivityForResult(downloadFileIntent, ConversationFragment.REQUEST_CODE_SAVE_FILE);
+            startActivityForResult(downloadFileIntent, ConversationFragment.REQUEST_CODE_SAVE_FILE);
+        } catch (Exception e) {
+            Log.i(TAG, "No app detected for saving files.");
+            File directory = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS);
+            if (!directory.exists()) {
+                directory.mkdirs();
+            }
+            writeToFile(android.net.Uri.fromFile(new File(directory, file.getDisplayName())));
+        }
     }
 
     @Override
     public void displayNetworkErrorPanel() {
-        showErrorPanel(R.string.error_no_network, null);
-    }
-
-    public void showErrorPanel(final int textResId, @Nullable View.OnClickListener clickListener) {
-        if (binding.errorMsgPane == null || binding.errorMsgPane.getVisibility() == View.VISIBLE) {
-            return;
-        }
-        binding.errorMsgPane.setVisibility(View.VISIBLE);
-        binding.errorMsgPane.setOnClickListener(clickListener);
-        if (binding.errorMsgTxt != null) {
-            binding.errorMsgTxt.setText(textResId);
+        if (binding != null) {
+            binding.errorMsgPane.setVisibility(View.VISIBLE);
+            binding.errorMsgPane.setOnClickListener(null);
+            binding.errorMsgPane.setText(R.string.error_no_network);
         }
     }
 
     @Override
     public void hideErrorPanel() {
-        if (binding.errorMsgPane == null || binding.errorMsgPane.getVisibility() == View.GONE) {
-            return;
+        if (binding != null) {
+            binding.errorMsgPane.setVisibility(View.GONE);
         }
-        binding.errorMsgPane.setVisibility(View.GONE);
     }
 
 }
