@@ -227,27 +227,12 @@ public class AccountService {
         public String address;
         public int state;
     }
-    public static class User {
-        public String username;
-        public String firstName;
-        public String lastName;
-        public String organization;
-        public String picture_b64;
-
-        public User(Map<String, String> m) {
-            username = m.get("username");
-            firstName = m.get("firstName");
-            lastName = m.get("lastName");
-            organization = m.get("organization");
-            picture_b64 = m.get("profilePicture");
-        }
-    }
     public static class UserSearchResult {
-        private String accountId;
-        private String query;
+        private final String accountId;
+        private final String query;
 
         public int state;
-        public List<User> results;
+        public List<CallContact> results;
 
         public UserSearchResult(String account, String query) {
             accountId = account;
@@ -264,8 +249,8 @@ public class AccountService {
 
         public List<Observable<SmartListViewModel>> getResultsViewModels() {
             List<Observable<SmartListViewModel>> vms = new ArrayList<>(results.size());
-            for (User user : results) {
-                vms.add(Observable.just(new SmartListViewModel(accountId, user)));
+            for (CallContact user : results) {
+                vms.add(Observable.just(new SmartListViewModel(accountId, user, null)));
             }
             return vms;
         }
@@ -502,6 +487,9 @@ public class AccountService {
     public Account getCurrentAccount() {
         return mCurrentAccount;
     }
+    public int getCurrentAccountIndex() {
+        return mAccountList.indexOf(mCurrentAccount);
+    }
 
     /**
      * Sets the current Account in the local cache (also sends a ACCOUNTS_CHANGED event)
@@ -653,16 +641,9 @@ public class AccountService {
     /**
      * @return Account Ids list from Daemon
      */
-    public List<String> getAccountList() {
-        try {
-            return mExecutor.submit(() -> {
-                Log.i(TAG, "getAccountList() running...");
-                return new ArrayList<>(Ringservice.getAccountList());
-            }).get();
-        } catch (Exception e) {
-            Log.e(TAG, "Error running getAccountList()", e);
-        }
-        return new ArrayList<>();
+    public Single<List<String>> getAccountList() {
+        return Single.fromCallable(() -> (List<String>)new ArrayList<>(Ringservice.getAccountList()))
+                .subscribeOn(Schedulers.from(mExecutor));
     }
 
     /**
@@ -977,7 +958,7 @@ public class AccountService {
     /**
      * Sets the account's credentials in the Daemon
      */
-    public void setCredentials(final String accountId, final List credentials) {
+    public void setCredentials(final String accountId, final List<Map<String, String>> credentials) {
         Log.i(TAG, "setCredentials() " + accountId);
         mExecutor.execute(() -> Ringservice.setCredentials(accountId, SwigNativeConverter.toSwig(credentials)));
     }
@@ -1313,6 +1294,16 @@ public class AccountService {
                 .subscribe(account::setProfile, e -> Log.e(TAG, "Error saving profile", e));
     }
 
+    void profileReceived(String accountId, String peerId, String vcardPath) {
+        Account account = getAccount(accountId);
+        if (account == null)
+            return;
+        Log.w(TAG, "profileReceived: " + accountId + ", " + peerId + ", " + vcardPath);
+        CallContact contact = account.getContactFromCache(peerId);
+        mVCardService.peerProfileReceived(accountId, peerId, new File(vcardPath))
+                .subscribe(profile -> contact.setProfile(profile.first, profile.second), e -> Log.e(TAG, "Error saving contact profile", e));
+    }
+
     void incomingAccountMessage(String accountId, String messageId, String callId, String from, Map<String, String> messages) {
         Log.d(TAG, "incomingAccountMessage: " + accountId + " " + messages.size());
         Message message = new Message();
@@ -1447,9 +1438,11 @@ public class AccountService {
     void registeredNameFound(String accountId, int state, String address, String name) {
         // Log.d(TAG, "registeredNameFound: " + accountId + ", " + state + ", " + name + ", " + address);
 
-        Account account = getAccount(accountId);
-        if (account != null) {
-            account.registeredNameFound(state, address, name);
+        if (!StringUtils.isEmpty(address)) {
+            Account account = getAccount(accountId);
+            if (account != null) {
+                account.registeredNameFound(state, address, name);
+            }
         }
 
         RegisteredName r = new RegisteredName();
@@ -1461,11 +1454,22 @@ public class AccountService {
     }
 
     public void userSearchEnded(String accountId, int state, String query, ArrayList<Map<String, String>> results) {
+        Account account = getAccount(accountId);
         UserSearchResult r = new UserSearchResult(accountId, query);
         r.state = state;
-        r.results = new ArrayList(results.size());//results.stream().map(item -> new User(item)).collect(Collectors.toList());
+        r.results = new ArrayList<>(results.size());
         for (Map<String, String> m : results) {
-            r.results.add(new User(m));
+            String uri = m.get("id");
+            String username = m.get("username");
+            String firstName = m.get("firstName");
+            String lastName = m.get("lastName");
+            String picture_b64 = m.get("profilePicture");
+            CallContact contact = account.getContactFromCache(uri);
+            if (contact != null) {
+                contact.setUsername(username);
+                contact.setProfile(firstName + " " + lastName, mVCardService.base64ToBitmap(picture_b64));
+                r.results.add(contact);
+            }
         }
         searchResultSubject.onNext(r);
     }
