@@ -41,9 +41,10 @@
 // Gnome client
 #include "native/pixbufmanipulator.h"
 #include "conversationpopupmenu.h"
+#include "utils/files.h"
 
-static constexpr const char* CALL_TARGET    = "CALL_TARGET";
-static constexpr int         CALL_TARGET_ID = 0;
+static constexpr const char* TEXT_URI_LIST_TARGET    = "text/uri-list";
+static constexpr int         TEXT_URI_LIST_TARGET_ID = 0;
 
 struct _ConversationsView
 {
@@ -57,6 +58,10 @@ struct _ConversationsViewClass
 
 typedef struct _ConversationsViewPrivate ConversationsViewPrivate;
 
+namespace { namespace details {
+class CppImpl;
+}}
+
 struct _ConversationsViewPrivate
 {
     AccountInfoPointer const *accountInfo_;
@@ -64,10 +69,13 @@ struct _ConversationsViewPrivate
     GtkWidget* popupMenu_;
 
     bool useDarkTheme {false};
+    details::CppImpl* cpp {nullptr};
 
     QMetaObject::Connection selection_updated;
     QMetaObject::Connection layout_changed;
     QMetaObject::Connection modelSortedConnection_;
+    QMetaObject::Connection searchChangedConnection_;
+    QMetaObject::Connection searchStatusChangedConnection_;
     QMetaObject::Connection conversationUpdatedConnection_;
     QMetaObject::Connection filterChangedConnection_;
     QMetaObject::Connection callChangedConnection_;
@@ -77,6 +85,22 @@ G_DEFINE_TYPE_WITH_PRIVATE(ConversationsView, conversations_view, GTK_TYPE_TREE_
 
 #define CONVERSATIONS_VIEW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CONVERSATIONS_VIEW_TYPE, ConversationsViewPrivate))
 
+namespace { namespace details {
+
+class CppImpl
+{
+public:
+    explicit CppImpl();
+
+    QString status;
+};
+
+CppImpl::CppImpl()
+    : status("")
+{}
+
+}}
+
 static void
 render_contact_photo(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
                      GtkCellRenderer *cell,
@@ -86,22 +110,32 @@ render_contact_photo(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
 {
     // Get active conversation
     auto path = gtk_tree_model_get_path(model, iter);
-    auto row = std::atoi(gtk_tree_path_to_string(path));
-    if (row == -1) return;
     auto priv = CONVERSATIONS_VIEW_GET_PRIVATE(self);
     if (!priv) return;
+    gchar *uid;
+
+    gtk_tree_model_get (model, iter,
+                        0 /* col# */, &uid /* data */,
+                        -1);
+
     try
     {
         // Draw first contact.
         // NOTE: We just draw the first contact, must change this for conferences when they will have their own object
-        auto conversationInfo = (*priv->accountInfo_)->conversationModel->filteredConversation(row);
-        auto contactInfo = (*priv->accountInfo_)->contactModel->getContact(conversationInfo.participants.front());
+        auto conversation = (*priv->accountInfo_)->conversationModel->getConversationForUID(uid);
+        auto isPresent = false;
+        auto isBanned = false;
+        if (not conversation.participants.empty()) {
+            auto contactInfo = (*priv->accountInfo_)->contactModel->getContact(conversation.participants.front());
+            isPresent = contactInfo.isPresent;
+            isBanned = contactInfo.isBanned;
+        }
         std::shared_ptr<GdkPixbuf> image;
         auto var_photo = GlobalInstances::pixmapManipulator().conversationPhoto(
-            conversationInfo,
+            conversation,
             **(priv->accountInfo_),
             QSize(50, 50),
-            contactInfo.isPresent
+            isPresent
         );
         image = var_photo.value<std::shared_ptr<GdkPixbuf>>();
 
@@ -111,12 +145,14 @@ render_contact_photo(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
         g_object_set(G_OBJECT(cell), "pixbuf", image.get(), NULL);
 
         // Banned contacts should be displayed with grey bg
-        g_object_set(G_OBJECT(cell), "cell-background", contactInfo.isBanned ? "#BDBDBD" : NULL, NULL);
+        g_object_set(G_OBJECT(cell), "cell-background", isBanned ? "#BDBDBD" : NULL, NULL);
     }
     catch (const std::exception&)
     {
-        g_warning("Can't get conversation at row %i", row);
+        g_warning("Can't get conversation %s", uid);
     }
+
+    g_free(uid);
 }
 
 static void
@@ -141,10 +177,6 @@ render_name_and_last_interaction(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
     auto priv = CONVERSATIONS_VIEW_GET_PRIVATE(treeview);
     if (!priv) return;
 
-    auto conversation = (*priv->accountInfo_)->conversationModel->filteredConversation(row);
-    auto contactUri = conversation.participants.front();
-    auto contactInfo = (*priv->accountInfo_)->contactModel->getContact(contactUri);
-
     auto grey = priv->useDarkTheme? "#bbb" : "#666";
 
     gtk_tree_model_get (model, iter,
@@ -156,7 +188,16 @@ render_name_and_last_interaction(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
                         -1);
 
     auto bestId = std::string(registeredName).empty() ? uri: registeredName;
-    if (contactInfo.isBanned) {
+
+    auto conversation = (*priv->accountInfo_)->conversationModel->getConversationForUID(uid);
+    auto isBanned = false;
+    if (not conversation.participants.empty()) {
+        auto contactUri = conversation.participants.front();
+        auto contactInfo = (*priv->accountInfo_)->contactModel->getContact(contactUri);
+        isBanned = contactInfo.isBanned;
+    }
+
+    if (isBanned) {
         // Contact is banned, display it clearly
         text = g_markup_printf_escaped(
             "<span font_weight=\"bold\">%s</span>\n<span size=\"smaller\" font_weight=\"bold\">Banned contact</span>",
@@ -191,7 +232,7 @@ render_name_and_last_interaction(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
     }
 
     // Banned contacts should be displayed with grey bg
-    g_object_set(G_OBJECT(cell), "cell-background", contactInfo.isBanned ? "#BDBDBD" : NULL, NULL);
+    g_object_set(G_OBJECT(cell), "cell-background", isBanned ? "#BDBDBD" : NULL, NULL);
 
     g_object_set(G_OBJECT(cell), "markup", text, NULL);
     g_free(text);
@@ -216,20 +257,35 @@ render_time(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
     auto path = gtk_tree_model_get_path(model, iter);
     auto row = std::atoi(gtk_tree_path_to_string(path));
     g_return_if_fail(row != -1);
+    gchar *uid;
 
-    try {
-        auto conversation = (*priv->accountInfo_)->conversationModel->filteredConversation(row);
+    gtk_tree_model_get (model, iter,
+                        0 /* col# */, &uid /* data */,
+                        -1);
+    if (g_strcmp0(uid, "") == 0) {
+        g_object_set(G_OBJECT(cell), "markup", "", NULL);
+        g_free(uid);
+        return;
+    }
+
+    try
+    {
+        // Draw first contact.
+        // NOTE: We just draw the first contact, must change this for conferences when they will have their own object
+        auto conversation = (*priv->accountInfo_)->conversationModel->getConversationForUID(uid);
+        if (conversation.participants.empty()) return;
         auto contactUri = conversation.participants.front();
         auto& contactInfo = (*priv->accountInfo_)->contactModel->getContact(contactUri);
 
         // Banned contacts should be displayed with grey bg
         g_object_set(G_OBJECT(cell), "cell-background", contactInfo.isBanned ? "#BDBDBD" : NULL, NULL);
 
-        auto callId = conversation.confId.empty() ? conversation.callId : conversation.confId;
-        if (!callId.empty()) {
+        auto callId = conversation.confId.isEmpty() ? conversation.callId : conversation.confId;
+        if (!callId.isEmpty()) {
             auto call = (*priv->accountInfo_)->callModel->getCall(callId);
             if (call.status != lrc::api::call::Status::ENDED) {
-                g_object_set(G_OBJECT(cell), "markup", lrc::api::call::to_string(call.status).c_str(), NULL);
+                g_object_set(G_OBJECT(cell), "markup", qUtf8Printable(lrc::api::call::to_string(call.status)), NULL);
+                g_free(uid);
                 return;
             }
         }
@@ -249,12 +305,14 @@ render_time(G_GNUC_UNUSED GtkTreeViewColumn *tree_column,
             g_object_set(G_OBJECT(cell), "markup", text, NULL);
 
             g_free(text);
+            g_free(uid);
             return;
         }
     } catch (const std::exception&) {
         g_warning("Can't get conversation at row %i", row);
     }
 
+    g_free(uid);
     g_object_set(G_OBJECT(cell), "markup", "", NULL);
 }
 
@@ -271,15 +329,15 @@ update_conversation(ConversationsView *self, const std::string& uid) {
         iterIsCorrect = gtk_tree_model_iter_nth_child (model, &iter, nullptr, idx);
         if (!iterIsCorrect)
             break;
-        gchar *ringId;
+        gchar *uidModel;
         gtk_tree_model_get (model, &iter,
-                            0 /* col# */, &ringId /* data */,
+                            0 /* col# */, &uidModel /* data */,
                             -1);
-        if(std::string(ringId) == uid) {
+        if(std::string(uid) == uidModel) {
             // Get informations
-            auto conversation = (*priv->accountInfo_)->conversationModel->filteredConversation(idx);
+            auto conversation = (*priv->accountInfo_)->conversationModel->getConversationForUID(uidModel);
             if (conversation.participants.empty()) {
-                g_free(ringId);
+                g_free(uidModel);
                 return;
             }
             auto contactUri = conversation.participants.front();
@@ -288,20 +346,21 @@ update_conversation(ConversationsView *self, const std::string& uid) {
                 conversation.interactions.at(conversation.lastMessageUid).body;
             std::replace(lastMessage.begin(), lastMessage.end(), '\n', ' ');
             auto alias = contactInfo.profileInfo.alias;
-            alias.erase(std::remove(alias.begin(), alias.end(), '\r'), alias.end());
+            alias.remove('\r');
             // Update iter
+
             gtk_list_store_set (GTK_LIST_STORE(model), &iter,
-                                0 /* col # */ , conversation.uid.c_str() /* celldata */,
-                                1 /* col # */ , alias.c_str() /* celldata */,
-                                2 /* col # */ , contactInfo.profileInfo.uri.c_str() /* celldata */,
-                                3 /* col # */ , contactInfo.registeredName.c_str() /* celldata */,
-                                4 /* col # */ , contactInfo.profileInfo.avatar.c_str() /* celldata */,
-                                5 /* col # */ , lastMessage.c_str() /* celldata */,
+                                0 /* col # */ , qUtf8Printable(conversation.uid) /* celldata */,
+                                1 /* col # */ , qUtf8Printable(alias) /* celldata */,
+                                2 /* col # */ , qUtf8Printable(contactInfo.profileInfo.uri) /* celldata */,
+                                3 /* col # */ , qUtf8Printable(contactInfo.registeredName) /* celldata */,
+                                4 /* col # */ , qUtf8Printable(contactInfo.profileInfo.avatar) /* celldata */,
+                                5 /* col # */ , qUtf8Printable(lastMessage) /* celldata */,
                                 -1 /* end */);
-            g_free(ringId);
+            g_free(uidModel);
             return;
         }
-        g_free(ringId);
+        g_free(uidModel);
         idx++;
     }
 }
@@ -321,49 +380,76 @@ create_and_fill_model(ConversationsView *self)
     if(!priv) GTK_TREE_MODEL (store);
     GtkTreeIter iter;
 
-    for (auto conversation : (*priv->accountInfo_)->conversationModel->allFilteredConversations()) {
-        if (conversation.participants.empty()) {
-            g_debug("Found conversation with empty list of participants - most likely the result of earlier bug.");
-            break;
-        }
-
-        auto contactUri = conversation.participants.front();
-        try {
-            auto contactInfo = (*priv->accountInfo_)->contactModel->getContact(contactUri);
-            auto lastMessage = conversation.interactions.empty() ? "" :
-                conversation.interactions.at(conversation.lastMessageUid).body;
-            std::replace(lastMessage.begin(), lastMessage.end(), '\n', ' ');
-            gtk_list_store_append (store, &iter);
-            auto alias = contactInfo.profileInfo.alias;
-            alias.erase(std::remove(alias.begin(), alias.end(), '\r'), alias.end());
-            gtk_list_store_set (store, &iter,
-                                0 /* col # */ , conversation.uid.c_str() /* celldata */,
-                                1 /* col # */ , alias.c_str() /* celldata */,
-                                2 /* col # */ , contactInfo.profileInfo.uri.c_str() /* celldata */,
-                                3 /* col # */ , contactInfo.registeredName.c_str() /* celldata */,
-                                4 /* col # */ , contactInfo.profileInfo.avatar.c_str() /* celldata */,
-                                5 /* col # */ , lastMessage.c_str() /* celldata */,
-                                -1 /* end */);
-        } catch (const std::out_of_range&) {
-            // ContactModel::getContact() exception
-        }
+    if (priv->cpp && !priv->cpp->status.isEmpty()) {
+        gtk_list_store_append (store, &iter);
+        gtk_list_store_set (store, &iter,
+                            0 /* col # */ , "" /* celldata */,
+                            1 /* col # */ , qUtf8Printable(priv->cpp->status) /* celldata */,
+                            2 /* col # */ , "" /* celldata */,
+                            3 /* col # */ , "" /* celldata */,
+                            4 /* col # */ , "" /* celldata */,
+                            5 /* col # */ , "" /* celldata */,
+                            -1 /* end */);
     }
 
+
+    auto fillModelLambda = [&](const auto& queue) {
+        for (auto conversation : queue) {
+            if (conversation.participants.empty()) {
+                g_debug("Found conversation with empty list of participants - most likely the result of earlier bug.");
+                break;
+            }
+
+            auto contactUri = conversation.participants.front();
+            try {
+                auto contactInfo = (*priv->accountInfo_)->contactModel->getContact(contactUri);
+                auto lastMessage = conversation.interactions.empty() ? "" :
+                    conversation.interactions.at(conversation.lastMessageUid).body;
+                std::replace(lastMessage.begin(), lastMessage.end(), '\n', ' ');
+                gtk_list_store_append (store, &iter);
+                auto alias = contactInfo.profileInfo.alias;
+                alias.remove('\r');
+                gtk_list_store_set (store, &iter,
+                                    0 /* col # */ , qUtf8Printable(conversation.uid) /* celldata */,
+                                    1 /* col # */ , qUtf8Printable(alias) /* celldata */,
+                                    2 /* col # */ , qUtf8Printable(contactInfo.profileInfo.uri) /* celldata */,
+                                    3 /* col # */ , qUtf8Printable(contactInfo.registeredName) /* celldata */,
+                                    4 /* col # */ , qUtf8Printable(contactInfo.profileInfo.avatar) /* celldata */,
+                                    5 /* col # */ , qUtf8Printable(lastMessage) /* celldata */,
+                                    -1 /* end */);
+            } catch (const std::out_of_range&) {
+                // ContactModel::getContact() exception
+            }
+        }
+    };
+
+    fillModelLambda((*priv->accountInfo_)->conversationModel->getAllSearchResults());
+    fillModelLambda((*priv->accountInfo_)->conversationModel->allFilteredConversations());
+
     return GTK_TREE_MODEL (store);
 }
 
 static void
 call_conversation(GtkTreeView *self,
-                  GtkTreePath *path,
+                  G_GNUC_UNUSED GtkTreePath *path,
                   G_GNUC_UNUSED GtkTreeViewColumn *column,
                   G_GNUC_UNUSED gpointer user_data)
 {
-    auto row = std::atoi(gtk_tree_path_to_string(path));
-    if (row == -1) return;
     auto priv = CONVERSATIONS_VIEW_GET_PRIVATE(self);
     if (!priv) return;
-    auto conversation = (*priv->accountInfo_)->conversationModel->filteredConversation(row);
-    (*priv->accountInfo_)->conversationModel->placeCall(conversation.uid);
+    GtkTreeIter iter;
+    GtkTreeModel *model = nullptr;
+    gchar *uid = nullptr;
+
+    auto selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(self));
+    if (!gtk_tree_selection_get_selected(selection, &model, &iter)) return;
+
+    gtk_tree_model_get(model, &iter,
+                       0, &uid,
+                       -1);
+
+    (*priv->accountInfo_)->conversationModel->placeCall(uid);
+    g_free(uid);
 }
 
 static void
@@ -399,7 +485,7 @@ select_conversation(GtkTreeSelection *selection, ConversationsView *self)
     gtk_tree_model_get(model, &iter,
                        0, &conversationUid,
                        -1);
-    (*priv->accountInfo_)->conversationModel->selectConversation(std::string(conversationUid));
+    (*priv->accountInfo_)->conversationModel->selectConversation(QString(conversationUid));
 }
 
 static void
@@ -419,66 +505,6 @@ show_popup_menu(ConversationsView *self, GdkEventButton *event)
         conversation_popup_menu_show(CONVERSATION_POPUP_MENU(priv->popupMenu_), event);
 }
 
-static void
-on_drag_data_get(GtkWidget        *treeview,
-                 G_GNUC_UNUSED GdkDragContext *context,
-                 GtkSelectionData *data,
-                 G_GNUC_UNUSED guint info,
-                 G_GNUC_UNUSED guint time,
-                 G_GNUC_UNUSED gpointer user_data)
-{
-    g_return_if_fail(IS_CONVERSATIONS_VIEW(treeview));
-
-    /* we always drag the selected row */
-    auto selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-    GtkTreeModel *model = NULL;
-    GtkTreeIter iter;
-
-    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-        auto path_str = gtk_tree_model_get_string_from_iter(model, &iter);
-
-        gtk_selection_data_set(data,
-                               gdk_atom_intern_static_string(CALL_TARGET),
-                               8, /* bytes */
-                               (guchar *)path_str,
-                               strlen(path_str) + 1);
-
-        g_free(path_str);
-    } else {
-        g_warning("drag selection not valid");
-    }
-}
-
-static gboolean
-on_drag_drop(GtkWidget      *treeview,
-             GdkDragContext *context,
-             gint            x,
-             gint            y,
-             guint           time,
-             G_GNUC_UNUSED gpointer user_data)
-{
-    g_return_val_if_fail(IS_CONVERSATIONS_VIEW(treeview), FALSE);
-
-    GtkTreePath *path = NULL;
-    GtkTreeViewDropPosition drop_pos;
-
-    if (gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW(treeview),
-                                          x, y, &path, &drop_pos)) {
-
-        GdkAtom target_type = gtk_drag_dest_find_target(treeview, context, NULL);
-
-        if (target_type != GDK_NONE) {
-            g_debug("can drop");
-            gtk_drag_get_data(treeview, context, target_type, time);
-            return TRUE;
-        }
-
-        gtk_tree_path_free(path);
-    }
-
-    return FALSE;
-}
-
 static gboolean
 on_drag_motion(GtkWidget      *treeview,
                GdkDragContext *context,
@@ -514,7 +540,7 @@ on_drag_data_received(GtkWidget        *treeview,
                       gint              x,
                       gint              y,
                       GtkSelectionData *data,
-                      G_GNUC_UNUSED guint info,
+                      guint             target_type,
                       guint             time,
                       G_GNUC_UNUSED gpointer user_data)
 {
@@ -527,6 +553,7 @@ on_drag_data_received(GtkWidget        *treeview,
     auto path_str_source = (gchar *)gtk_selection_data_get_data(data);
     auto type = gtk_selection_data_get_data_type(data);
     g_debug("data type: %s", gdk_atom_name(type));
+
     if (path_str_source && strlen(path_str_source) > 0) {
         g_debug("source path: %s", path_str_source);
 
@@ -535,30 +562,27 @@ on_drag_data_received(GtkWidget        *treeview,
         if (gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW(treeview), x, y, &dest_path, NULL)) {
             auto model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
 
-            GtkTreeIter source, dest;
-            gtk_tree_model_get_iter_from_string(model, &source, path_str_source);
-            gtk_tree_model_get_iter(model, &dest, dest_path);
+            switch (target_type) {
+                case TEXT_URI_LIST_TARGET_ID: {
+                    GtkTreeIter dest;
+                    gtk_tree_model_get_iter(model, &dest, dest_path);
+                    gchar *conversationUid = nullptr;
+                    gtk_tree_model_get(model, &dest, 0, &conversationUid, -1);
 
-            gchar *conversationUidSrc = nullptr;
-            gchar *conversationUidDest = nullptr;
+                    foreach_file(path_str_source, [&](const char* file) {
+                        (*priv->accountInfo_)->conversationModel->sendFile(
+                            conversationUid, file, g_path_get_basename(file));
+                    });
 
-            gtk_tree_model_get(model, &source,
-                               0, &conversationUidSrc,
-                               -1);
-            gtk_tree_model_get(model, &dest,
-                               0, &conversationUidDest,
-                               -1);
-
-            (*priv->accountInfo_)->conversationModel->joinConversations(
-                conversationUidSrc,
-                conversationUidDest
-            );
-
-            gtk_tree_path_free(dest_path);
-            g_free(conversationUidSrc);
-            g_free(conversationUidDest);
-
-            success = TRUE;
+                    gtk_tree_path_free(dest_path);
+                    g_free(conversationUid);
+                    success = TRUE;
+                    break;
+                }
+                default:
+                    g_warning("unhandled drag and drop target type");
+                    success = FALSE;
+            }
         }
     }
 
@@ -568,12 +592,13 @@ on_drag_data_received(GtkWidget        *treeview,
 static void
 build_conversations_view(ConversationsView *self)
 {
-    auto priv = CONVERSATIONS_VIEW_GET_PRIVATE(self);
+    auto* priv = CONVERSATIONS_VIEW_GET_PRIVATE(self);
     gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(self), FALSE);
 
     auto model = create_and_fill_model(self);
     gtk_tree_view_set_model(GTK_TREE_VIEW(self),
                             GTK_TREE_MODEL(model));
+    g_object_unref(model);
 
     gtk_tree_view_set_enable_search(GTK_TREE_VIEW(self), false);
 
@@ -628,12 +653,68 @@ build_conversations_view(ConversationsView *self)
 
         gtk_tree_view_set_model(GTK_TREE_VIEW(self),
                                 GTK_TREE_MODEL(model));
+        g_object_unref(model);
+    });
+
+
+    priv->searchChangedConnection_ = QObject::connect(
+    &*(*priv->accountInfo_)->conversationModel,
+    &lrc::api::ConversationModel::searchResultUpdated,
+    [self] () {
+        auto model = create_and_fill_model(self);
+
+        gtk_tree_view_set_model(GTK_TREE_VIEW(self),
+                                GTK_TREE_MODEL(model));
+        g_object_unref(model);
+    });
+    priv->searchStatusChangedConnection_ = QObject::connect(
+    &*(*priv->accountInfo_)->conversationModel,
+    &lrc::api::ConversationModel::searchStatusChanged,
+    [self] (const QString& status) {
+        auto priv = CONVERSATIONS_VIEW_GET_PRIVATE(self);
+        if (!priv or !priv->cpp) return;
+        priv->cpp->status = status;
+        if (status.isEmpty()) return;
+
+        auto model = gtk_tree_view_get_model(GTK_TREE_VIEW(self));
+        GtkTreeIter iter;
+
+        auto addItem = true;
+        if (gtk_tree_model_iter_children(model, &iter, nullptr)) {
+            gchar *conversationUid = nullptr;
+            gtk_tree_model_get(model, &iter, 0, &conversationUid, -1);
+            if (g_strcmp0(conversationUid, "") == 0) {
+                addItem = false;
+                gtk_list_store_set (GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(self))), &iter,
+                                    0 /* col # */ , "" /* celldata */,
+                                    1 /* col # */ , qUtf8Printable(status) /* celldata */,
+                                    2 /* col # */ , "" /* celldata */,
+                                    3 /* col # */ , "" /* celldata */,
+                                    4 /* col # */ , "" /* celldata */,
+                                    5 /* col # */ , "" /* celldata */,
+                                    -1 /* end */);
+            }
+            g_free(conversationUid);
+        }
+
+        if (addItem) {
+            gtk_list_store_append (GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(self))), &iter);
+            gtk_list_store_set (GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(self))), &iter,
+                                0 /* col # */ , "" /* celldata */,
+                                1 /* col # */ , qUtf8Printable(status) /* celldata */,
+                                2 /* col # */ , "" /* celldata */,
+                                3 /* col # */ , "" /* celldata */,
+                                4 /* col # */ , "" /* celldata */,
+                                5 /* col # */ , "" /* celldata */,
+                                -1 /* end */);
+        }
+
     });
     priv->conversationUpdatedConnection_ = QObject::connect(
     &*(*priv->accountInfo_)->conversationModel,
     &lrc::api::ConversationModel::conversationUpdated,
-    [self] (const std::string& uid) {
-        update_conversation(self, uid);
+    [self] (const QString& uid) {
+        update_conversation(self, uid.toStdString());
     });
 
     priv->filterChangedConnection_ = QObject::connect(
@@ -644,12 +725,13 @@ build_conversations_view(ConversationsView *self)
 
         gtk_tree_view_set_model(GTK_TREE_VIEW(self),
                                 GTK_TREE_MODEL(model));
+        g_object_unref(model);
     });
 
     priv->callChangedConnection_ = QObject::connect(
     &*(*priv->accountInfo_)->callModel,
     &lrc::api::NewCallModel::callStatusChanged,
-    [self, priv] (const std::string&) {
+    [self, priv] (const QString&) {
         // retrieve currently selected conversation
         GtkTreeIter iter;
         GtkTreeModel *model = nullptr;
@@ -662,9 +744,12 @@ build_conversations_view(ConversationsView *self)
         // create updated model
         auto new_model = create_and_fill_model(self);
         gtk_tree_view_set_model(GTK_TREE_VIEW(self), GTK_TREE_MODEL(new_model));
+        g_object_unref(new_model);
 
         // make sure conversation remains selected
         conversations_view_select_conversation(self, conversationUid);
+
+        g_free(conversationUid);
     });
 
     gtk_widget_show_all(GTK_WIDGET(self));
@@ -680,20 +765,18 @@ build_conversations_view(ConversationsView *self)
     g_signal_connect_swapped(self, "button-press-event", G_CALLBACK(show_popup_menu), self);
 
     /* drag and drop */
-    static GtkTargetEntry targetentries[] = {
-        { (gchar *)CALL_TARGET, GTK_TARGET_SAME_WIDGET, CALL_TARGET_ID },
+    static GtkTargetEntry target_entries[] = {
+        { (gchar *)TEXT_URI_LIST_TARGET, GTK_TARGET_OTHER_APP, TEXT_URI_LIST_TARGET_ID },
     };
 
-    gtk_tree_view_enable_model_drag_source(GTK_TREE_VIEW(self),
-        GDK_BUTTON1_MASK, targetentries, 1, (GdkDragAction)(GDK_ACTION_DEFAULT | GDK_ACTION_MOVE));
-
     gtk_tree_view_enable_model_drag_dest(GTK_TREE_VIEW(self),
-        targetentries, 1, GDK_ACTION_DEFAULT);
+        target_entries, 1,
+        (GdkDragAction)(GDK_ACTION_DEFAULT | GDK_ACTION_COPY));
 
-    g_signal_connect(self, "drag-data-get", G_CALLBACK(on_drag_data_get), nullptr);
-    g_signal_connect(self, "drag-drop", G_CALLBACK(on_drag_drop), nullptr);
     g_signal_connect(self, "drag-motion", G_CALLBACK(on_drag_motion), nullptr);
-    g_signal_connect(self, "drag_data_received", G_CALLBACK(on_drag_data_received), nullptr);
+    g_signal_connect(self, "drag-data-received", G_CALLBACK(on_drag_data_received), nullptr);
+
+    priv->cpp = new details::CppImpl();
 }
 
 static void
@@ -705,11 +788,15 @@ conversations_view_dispose(GObject *object)
     QObject::disconnect(priv->selection_updated);
     QObject::disconnect(priv->layout_changed);
     QObject::disconnect(priv->modelSortedConnection_);
+    QObject::disconnect(priv->searchChangedConnection_);
+    QObject::disconnect(priv->searchStatusChangedConnection_);
     QObject::disconnect(priv->conversationUpdatedConnection_);
     QObject::disconnect(priv->filterChangedConnection_);
     QObject::disconnect(priv->callChangedConnection_);
 
     gtk_widget_destroy(priv->popupMenu_);
+    delete priv->cpp;
+    priv->cpp = nullptr;
 
     G_OBJECT_CLASS(conversations_view_parent_class)->dispose(object);
 }
@@ -777,11 +864,10 @@ conversations_view_select_conversation(ConversationsView *self, const std::strin
     }
 }
 
-int
+std::string
 conversations_view_get_current_selected(ConversationsView *self)
 {
-
-    g_return_val_if_fail(IS_CONVERSATIONS_VIEW(self), -1);
+    g_return_val_if_fail(IS_CONVERSATIONS_VIEW(self), "");
 
     /* we always drag the selected row */
     auto selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(self));
@@ -789,11 +875,16 @@ conversations_view_get_current_selected(ConversationsView *self)
     GtkTreeIter iter;
 
     if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
-        auto path = gtk_tree_model_get_path(model, &iter);
-        auto idx = gtk_tree_path_get_indices(path);
-        return idx[0];
+        gchar* uid;
+        gtk_tree_model_get(model, &iter,
+                        0, &uid,
+                        -1);
+        if (!uid) return {};
+        std::string result = uid;
+        g_free(uid);
+        return result;
     }
-    return -1;
+    return {};
 }
 
 void
