@@ -87,6 +87,7 @@ using yaml_utils::parseValue;
 using yaml_utils::parseVectorMap;
 using sip_utils::CONST_PJ_STR;
 
+static constexpr int KEEP_ALIVE_DELAY = 10;                       // seconds
 static constexpr int MIN_REGISTRATION_TIME = 60;                  // seconds
 static constexpr unsigned DEFAULT_REGISTRATION_TIME = 3600;       // seconds
 static constexpr unsigned REGISTRATION_FIRST_RETRY_INTERVAL = 60; // seconds
@@ -142,7 +143,7 @@ SIPAccount::SIPAccount(const std::string& accountID, bool presenceEnabled)
     , tlsRequireClientCertificate_(true)
     , tlsNegotiationTimeoutSec_("2")
     , registrationStateDetailed_()
-    , keepAliveEnabled_(false)
+    , keepAliveEnabled_(true)
     , keepAliveTimer_()
     , keepAliveTimerActive_(false)
     , receivedParameter_("")
@@ -178,7 +179,7 @@ SIPAccount::newIncomingCall(const std::string& from UNUSED,
                             const std::shared_ptr<SipTransport>&)
 {
     auto& manager = Manager::instance();
-    return manager.callFactory.newCall<SIPCall, SIPAccount>(*this,
+    return manager.callFactory.newCall<SIPCall, SIPAccount>(shared(),
                                                             manager.getNewCallID(),
                                                             Call::CallType::INCOMING,
                                                             details);
@@ -186,7 +187,7 @@ SIPAccount::newIncomingCall(const std::string& from UNUSED,
 
 template<>
 std::shared_ptr<SIPCall>
-SIPAccount::newOutgoingCall(const std::string& toUrl,
+SIPAccount::newOutgoingCall(std::string_view toUrl,
                             const std::map<std::string, std::string>& volatileCallDetails)
 {
     std::string to;
@@ -195,7 +196,7 @@ SIPAccount::newOutgoingCall(const std::string& toUrl,
     JAMI_DBG() << *this << "Calling SIP peer " << toUrl;
 
     auto& manager = Manager::instance();
-    auto call = manager.callFactory.newCall<SIPCall, SIPAccount>(*this,
+    auto call = manager.callFactory.newCall<SIPCall, SIPAccount>(shared(),
                                                                  manager.getNewCallID(),
                                                                  Call::CallType::OUTGOING,
                                                                  volatileCallDetails);
@@ -222,7 +223,7 @@ SIPAccount::newOutgoingCall(const std::string& toUrl,
         // FIXME : for now, use the same address family as the SIP transport
         family = pjsip_transport_type_get_af(getTransportType());
 
-        JAMI_DBG("UserAgent: New registered account call to %s", toUrl.c_str());
+        JAMI_DBG("UserAgent: New registered account call to %.*s", (int) toUrl.size(), toUrl.data());
     }
 
     auto toUri = getToUri(to);
@@ -341,7 +342,7 @@ SIPAccount::getTransportSelector()
 }
 
 std::shared_ptr<Call>
-SIPAccount::newOutgoingCall(const std::string& toUrl,
+SIPAccount::newOutgoingCall(std::string_view toUrl,
                             const std::map<std::string, std::string>& volatileCallDetails)
 {
     return newOutgoingCall<SIPCall>(toUrl, volatileCallDetails);
@@ -353,13 +354,12 @@ SIPAccount::SIPStartCall(std::shared_ptr<SIPCall>& call)
     // Add Ice headers to local SDP if ice transport exist
     call->setupLocalSDPFromIce();
 
-    std::string toUri(call->getPeerNumber()); // expecting a fully well formed sip uri
-
-    pj_str_t pjTo = pj_str((char*) toUri.c_str());
+    const std::string& toUri(call->getPeerNumber()); // expecting a fully well formed sip uri
+    pj_str_t pjTo = sip_utils::CONST_PJ_STR(toUri);
 
     // Create the from header
     std::string from(getFromUri());
-    pj_str_t pjFrom = pj_str((char*) from.c_str());
+    pj_str_t pjFrom = sip_utils::CONST_PJ_STR(from);
 
     auto transport = call->getTransport();
     if (!transport) {
@@ -409,6 +409,11 @@ SIPAccount::SIPStartCall(std::shared_ptr<SIPCall>& call)
         return false;
     }
 
+    setUpTransmissionData(tdata, tp_sel.u.transport->key.type);
+
+    // Add user-agent header
+    sip_utils::addUserAgenttHeader(getUserAgentName(), tdata);
+
     if (pjsip_inv_send_msg(call->inv.get(), tdata) != PJ_SUCCESS) {
         JAMI_ERR("Unable to send invite message for this call");
         return false;
@@ -840,7 +845,7 @@ SIPAccount::doRegister2_()
 
         // Dropping current calls already using the transport is currently required
         // with TLS.
-        freeAccount();
+        hangupCalls();
         initTlsConfiguration();
 
         if (!tlsListener_) {
@@ -906,8 +911,6 @@ SIPAccount::doUnregister(std::function<void(bool)> released_cb)
     std::unique_lock<std::mutex> lock(configurationMutex_);
 
     tlsListener_.reset();
-    if (transport_)
-        setTransport();
 
     if (!isIP2IP()) {
         try {
@@ -917,6 +920,10 @@ SIPAccount::doUnregister(std::function<void(bool)> released_cb)
         }
     }
 
+    if (transport_)
+        setTransport();
+    resetAutoRegistration();
+
     lock.unlock();
     if (released_cb)
         released_cb(not isIP2IP());
@@ -953,24 +960,25 @@ SIPAccount::startKeepAliveTimer()
     // make sure here we have an entirely new timer
     memset(&keepAliveTimer_, 0, sizeof(pj_timer_entry));
 
-    pj_time_val keepAliveDelay_;
-    keepAliveTimer_.cb = &SIPAccount::keepAliveRegistrationCb;
-    keepAliveTimer_.user_data = this;
+    keepAliveTimer_.cb = [](pj_timer_heap_t* /*th*/, pj_timer_entry* te) {
+        if (auto sipAccount = static_cast<std::weak_ptr<SIPAccount>*>(te->user_data)->lock())
+            sipAccount->keepAliveRegistrationCb();
+    };
+    if (not keepAliveTimer_.user_data)
+        keepAliveTimer_.user_data = new std::weak_ptr<SIPAccount>(weak());
     keepAliveTimer_.id = timerIdDist_(rand);
 
     // expiration may be undetermined during the first registration request
+    pj_time_val keepAliveDelay_;
     if (registrationExpire_ == 0) {
         JAMI_DBG("Registration Expire: 0, taking 60 instead");
-        keepAliveDelay_.sec = 3600;
+        keepAliveDelay_.sec = MIN_REGISTRATION_TIME;
     } else {
         JAMI_DBG("Registration Expire: %d", registrationExpire_);
-        keepAliveDelay_.sec = registrationExpire_ + MIN_REGISTRATION_TIME;
+        keepAliveDelay_.sec = registrationExpire_ + KEEP_ALIVE_DELAY;
     }
-
     keepAliveDelay_.msec = 0;
-
     keepAliveTimerActive_ = true;
-
     link_.registerKeepAliveTimer(keepAliveTimer_, keepAliveDelay_);
 }
 
@@ -983,6 +991,10 @@ SIPAccount::stopKeepAliveTimer()
                  getAccountID().c_str());
         keepAliveTimerActive_ = false;
         link_.cancelKeepAliveTimer(keepAliveTimer_);
+        if (keepAliveTimer_.user_data) {
+            delete ((std::weak_ptr<SIPAccount>*) keepAliveTimer_.user_data);
+            keepAliveTimer_.user_data = nullptr;
+        }
     }
 }
 
@@ -1006,10 +1018,10 @@ SIPAccount::sendRegister()
 
     // Generate the FROM header
     std::string from(getFromUri());
-    pj_str_t pjFrom {(char*) from.data(), (pj_ssize_t) from.size()};
+    pj_str_t pjFrom(sip_utils::CONST_PJ_STR(from));
 
     // Get the received header
-    std::string received(getReceivedParameter());
+    const std::string& received(getReceivedParameter());
 
     // Get the contact header
     const pj_str_t pjContact(getContactHeader());
@@ -1052,37 +1064,28 @@ SIPAccount::sendRegister()
 
     pjsip_hdr hdr_list;
     pj_list_init(&hdr_list);
-    std::string useragent(getUserAgentName());
-    auto pJuseragent = CONST_PJ_STR(useragent);
+    auto pjUserAgent = CONST_PJ_STR(getUserAgentName());
     constexpr pj_str_t STR_USER_AGENT = CONST_PJ_STR("User-Agent");
 
     pjsip_generic_string_hdr* h = pjsip_generic_string_hdr_create(link_.getPool(),
                                                                   &STR_USER_AGENT,
-                                                                  &pJuseragent);
+                                                                  &pjUserAgent);
     pj_list_push_back(&hdr_list, (pjsip_hdr*) h);
     pjsip_regc_add_headers(regc, &hdr_list);
 
     pjsip_tx_data* tdata;
-    if (pjsip_regc_register(regc, PJ_TRUE, &tdata) != PJ_SUCCESS)
+    if (pjsip_regc_register(regc, PJ_FALSE, &tdata) != PJ_SUCCESS)
         throw VoipLinkException("Unable to initialize transaction data for account registration");
 
     const pjsip_tpselector tp_sel = getTransportSelector();
     if (pjsip_regc_set_transport(regc, &tp_sel) != PJ_SUCCESS)
         throw VoipLinkException("Unable to set transport");
 
-    if (hostIp_) {
-        auto ai = &tdata->dest_info;
-        ai->name = pj_strdup3(tdata->pool, hostname_.c_str());
-        ai->addr.count = 1;
-        ai->addr.entry[0].type = (pjsip_transport_type_e) tp_sel.u.transport->key.type;
-        pj_memcpy(&ai->addr.entry[0].addr, hostIp_.pjPtr(), sizeof(pj_sockaddr));
-        ai->addr.entry[0].addr_len = hostIp_.getLength();
-        ai->cur_addr = 0;
-    }
+    setUpTransmissionData(tdata, tp_sel.u.transport->key.type);
 
     // pjsip_regc_send increment the transport ref count by one,
     if ((status = pjsip_regc_send(regc, tdata)) != PJ_SUCCESS) {
-        JAMI_ERR("pjsip_regc_init failed with error %d: %s",
+        JAMI_ERR("pjsip_regc_send failed with error %d: %s",
                  status,
                  sip_utils::sip_strerror(status).c_str());
         throw VoipLinkException("Unable to send account registration request");
@@ -1091,6 +1094,31 @@ SIPAccount::sendRegister()
     setRegistrationInfo(regc);
 }
 
+void
+SIPAccount::setUpTransmissionData(pjsip_tx_data* tdata, long transportKeyType)
+{
+    if (hasServiceRoute()) {
+        IpAddr proxyServer {getServiceRoute()};
+        if (proxyServer.getPort() == 0)
+            proxyServer.setPort(sip_utils::DEFAULT_SIP_PORT);
+        auto ai = &tdata->dest_info;
+        ai->name = pj_strdup3(tdata->pool, hostname_.c_str());
+        ai->addr.count = 1;
+        ai->addr.entry[0].type = (pjsip_transport_type_e) transportKeyType;
+        pj_memcpy(&ai->addr.entry[0].addr, proxyServer.pjPtr(), sizeof(pj_sockaddr));
+        ai->addr.entry[0].addr_len = proxyServer.getLength();
+        ai->cur_addr = 0;
+    } else if (hostIp_) {
+        auto ai = &tdata->dest_info;
+        ai->name = pj_strdup3(tdata->pool, hostname_.c_str());
+        ai->addr.count = 1;
+        ai->addr.entry[0].type = (pjsip_transport_type_e) transportKeyType;
+        pj_memcpy(&ai->addr.entry[0].addr, hostIp_.pjPtr(), sizeof(pj_sockaddr));
+        ai->addr.entry[0].addr_len = hostIp_.getLength();
+        ai->cur_addr = 0;
+    }
+}
+
 void
 SIPAccount::onRegister(pjsip_regc_cbparam* param)
 {
@@ -1201,6 +1229,12 @@ SIPAccount::sendUnregister()
     if (pjsip_regc_unregister(regc, &tdata) != PJ_SUCCESS)
         throw VoipLinkException("Unable to unregister sip account");
 
+    const pjsip_tpselector tp_sel = getTransportSelector();
+    if (pjsip_regc_set_transport(regc, &tp_sel) != PJ_SUCCESS)
+        throw VoipLinkException("Unable to set transport");
+
+    setUpTransmissionData(tdata, tp_sel.u.transport->key.type);
+
     pj_status_t status;
     if ((status = pjsip_regc_send(regc, tdata)) != PJ_SUCCESS) {
         JAMI_ERR("pjsip_regc_send failed with error %d: %s",
@@ -1345,19 +1379,19 @@ SIPAccount::loadConfig()
 }
 
 bool
-SIPAccount::fullMatch(const std::string& username, const std::string& hostname) const
+SIPAccount::fullMatch(std::string_view username, std::string_view hostname) const
 {
     return userMatch(username) and hostnameMatch(hostname);
 }
 
 bool
-SIPAccount::userMatch(const std::string& username) const
+SIPAccount::userMatch(std::string_view username) const
 {
     return !username.empty() and username == username_;
 }
 
 bool
-SIPAccount::hostnameMatch(const std::string& hostname) const
+SIPAccount::hostnameMatch(std::string_view hostname) const
 {
     if (hostname == hostname_)
         return true;
@@ -1367,7 +1401,7 @@ SIPAccount::hostnameMatch(const std::string& hostname) const
 }
 
 bool
-SIPAccount::proxyMatch(const std::string& hostname) const
+SIPAccount::proxyMatch(std::string_view hostname) const
 {
     if (hostname == serviceRoute_)
         return true;
@@ -1516,7 +1550,7 @@ SIPAccount::getContactHeader(pjsip_transport* t)
         auto success = link_.findLocalAddressFromSTUN(t, &stunServerName_, stunPort_, address, port);
         if (not success)
             emitSignal<DRing::ConfigurationSignal::StunStatusFailed>(getAccountID());
-        setPublishedAddress(address);
+        setPublishedAddress({address});
         publishedPort_ = port;
         usePublishedAddressPortInVIA();
     } else {
@@ -1609,29 +1643,22 @@ SIPAccount::getSupportedTlsProtocols()
 }
 
 void
-SIPAccount::keepAliveRegistrationCb(UNUSED pj_timer_heap_t* th, pj_timer_entry* te)
+SIPAccount::keepAliveRegistrationCb()
 {
-    SIPAccount* sipAccount = static_cast<SIPAccount*>(te->user_data);
-
-    if (sipAccount == nullptr) {
-        JAMI_ERR("SIP account is nullptr while registering a new keep alive timer");
-        return;
-    }
-
-    JAMI_ERR("Keep alive registration callback for account %s", sipAccount->getAccountID().c_str());
+    JAMI_ERR("Keep alive registration callback for account %s", getAccountID().c_str());
 
     // IP2IP default does not require keep-alive
-    if (sipAccount->isIP2IP())
+    if (isIP2IP())
         return;
 
     // TLS is connection oriented and does not require keep-alive
-    if (sipAccount->isTlsEnabled())
+    if (isTlsEnabled())
         return;
 
-    sipAccount->stopKeepAliveTimer();
+    stopKeepAliveTimer();
 
-    if (sipAccount->isRegistered())
-        sipAccount->doRegister();
+    if (isRegistered())
+        doRegister();
 }
 
 void
@@ -1724,14 +1751,6 @@ SIPAccount::setRegistrationState(RegistrationState state,
     SIPAccountBase::setRegistrationState(state, details_code, details_str);
 }
 
-std::string
-SIPAccount::getUserAgentName() const
-{
-    if (not hasCustomUserAgent_ or userAgent_.empty())
-        return DEFAULT_USER_AGENT;
-    return userAgent_;
-}
-
 std::map<std::string, std::string>
 SIPAccount::getTlsSettings() const
 {
@@ -1811,21 +1830,29 @@ SIPAccount::supportPresence(int function, bool enabled)
 }
 
 MatchRank
-SIPAccount::matches(const std::string& userName, const std::string& server) const
+SIPAccount::matches(std::string_view userName, std::string_view server) const
 {
     if (fullMatch(userName, server)) {
-        JAMI_DBG("Matching account id in request is a fullmatch %s@%s",
-                 userName.c_str(),
-                 server.c_str());
+        JAMI_DBG("Matching account id in request is a fullmatch %.*s@%.*s",
+                 (int) userName.size(),
+                 userName.data(),
+                 (int) server.size(),
+                 server.data());
         return MatchRank::FULL;
     } else if (hostnameMatch(server)) {
-        JAMI_DBG("Matching account id in request with hostname %s", server.c_str());
+        JAMI_DBG("Matching account id in request with hostname %.*s",
+                 (int) server.size(),
+                 server.data());
         return MatchRank::PARTIAL;
     } else if (userMatch(userName)) {
-        JAMI_DBG("Matching account id in request with username %s", userName.c_str());
+        JAMI_DBG("Matching account id in request with username %.*s",
+                 (int) userName.size(),
+                 userName.data());
         return MatchRank::PARTIAL;
     } else if (proxyMatch(server)) {
-        JAMI_DBG("Matching account id in request with proxy %s", server.c_str());
+        JAMI_DBG("Matching account id in request with proxy %.*s",
+                 (int) server.size(),
+                 server.data());
         return MatchRank::PARTIAL;
     } else {
         return MatchRank::NONE;
@@ -1846,6 +1873,10 @@ SIPAccount::resetAutoRegistration()
 {
     auto_rereg_.active = PJ_FALSE;
     auto_rereg_.attempt_cnt = 0;
+    if (auto_rereg_.timer.user_data) {
+        delete ((std::weak_ptr<SIPAccount>*) auto_rereg_.timer.user_data);
+        auto_rereg_.timer.user_data = nullptr;
+    }
 }
 
 /* Update NAT address from the REGISTER response */
@@ -2064,9 +2095,11 @@ SIPAccount::scheduleReregistration()
 
     /* Set up timer for reregistration */
     auto_rereg_.timer.cb = [](pj_timer_heap_t* /*th*/, pj_timer_entry* te) {
-        static_cast<SIPAccount*>(te->user_data)->autoReregTimerCb();
+        if (auto sipAccount = static_cast<std::weak_ptr<SIPAccount>*>(te->user_data)->lock())
+            sipAccount->autoReregTimerCb();
     };
-    auto_rereg_.timer.user_data = this;
+    if (not auto_rereg_.timer.user_data)
+        auto_rereg_.timer.user_data = new std::weak_ptr<SIPAccount>(weak());
 
     /* Reregistration attempt. The first attempt will be done sooner */
     pj_time_val delay;
@@ -2133,8 +2166,8 @@ SIPAccount::sendTextMessage(const std::string& to,
 
     constexpr pjsip_method msg_method = {PJSIP_OTHER_METHOD, CONST_PJ_STR("MESSAGE")};
     std::string from(getFromUri());
-    pj_str_t pjFrom = pj_str((char*) from.c_str());
-    pj_str_t pjTo = pj_str((char*) toUri.c_str());
+    pj_str_t pjFrom = sip_utils::CONST_PJ_STR(from);
+    pj_str_t pjTo = sip_utils::CONST_PJ_STR(toUri);
 
     /* Create request. */
     pjsip_tx_data* tdata;
@@ -2168,15 +2201,8 @@ SIPAccount::sendTextMessage(const std::string& to,
         pjsip_date_hdr_create(tdata->pool, &key, pj_cstr(&date_str, date)));
     pjsip_msg_add_hdr(tdata->msg, hdr);
 
-    /* Add user agent header. */
-    pjsip_hdr* hdr_list;
-    auto pJuseragent = CONST_PJ_STR(getUserAgentName());
-    constexpr pj_str_t STR_USER_AGENT = CONST_PJ_STR("User-Agent");
-
-    // Add Header
-    hdr_list = reinterpret_cast<pjsip_hdr*>(
-        pjsip_user_agent_hdr_create(tdata->pool, &STR_USER_AGENT, &pJuseragent));
-    pjsip_msg_add_hdr(tdata->msg, hdr_list);
+    // Add user-agent header
+    sip_utils::addUserAgenttHeader(getUserAgentName(), tdata);
 
     // Set input token into callback
     std::unique_ptr<ctx> t {new ctx(new pjsip_auth_clt_sess)};
@@ -2212,6 +2238,8 @@ SIPAccount::sendTextMessage(const std::string& to,
         return;
     }
 
+    setUpTransmissionData(tdata, tp_sel.u.transport->key.type);
+
     im::fillPJSIPMessageBody(*tdata, payloads);
 
     // Send message request with callback SendMessageOnComplete
@@ -2255,6 +2283,8 @@ SIPAccount::onComplete(void* token, pjsip_event* event)
             cseq_hdr->cseq += 1;
 
             // Resend request
+            auto to = c->to;
+            auto id = c->id;
             status = pjsip_endpt_send_request(acc->link_.getEndpoint(),
                                               new_request,
                                               -1,
@@ -2263,7 +2293,7 @@ SIPAccount::onComplete(void* token, pjsip_event* event)
 
             if (status != PJ_SUCCESS) {
                 JAMI_ERR("Unable to send request: %s", sip_utils::sip_strerror(status).c_str());
-                acc->messageEngine_.onMessageSent(c->to, c->id, false);
+                acc->messageEngine_.onMessageSent(to, id, false);
             }
             return;
         } else {
